local anim8 = require "libraries/anim8"
local flux = require "libraries/flux"

enemies = {}

function enemies:load()

	math.random(os.time())

	self.width = 120
	self.height = 80
	
	self.x = (screenWidth - self.width) / 2
	self.y = 420

	self.img_left = love.graphics.newImage("assets/enemy/knight/enemy_left.png")
	self.img_right = love.graphics.newImage("assets/enemy/knight/enemy_right.png")

	self.img_eye_left = love.graphics.newImage("assets/enemy/eyes/eye_left.png")
	self.img_eye_right = love.graphics.newImage("assets/enemy/eyes/eye_right.png")
	self.img_eye_projectiles = love.graphics.newImage("assets/enemy/eyes/projectile_sprite.png")

	self.grid_eye_left = anim8.newGrid(150, 150, self.img_eye_left:getWidth(), self.img_eye_left:getHeight())
	self.grid_eye_right = anim8.newGrid(150, 150, self.img_eye_right:getWidth(), self.img_eye_right:getHeight())
	self.grid_eye_projectile = anim8.newGrid(48, 48, self.img_eye_projectiles:getWidth(), self.img_eye_projectiles:getHeight())

	self.grid_left = anim8.newGrid(self.width, self.height, self.img_left:getWidth(), self.img_left:getHeight())
	self.grid_right = anim8.newGrid(self.width, self.height, self.img_right:getWidth(), self.img_right:getHeight())

	self.img_projectile = love.graphics.newQuad(0, 0, 48, 48, self.img_eye_projectiles:getDimensions())

	self.animations_eye = {}
	self.animations_eye.left_fly = anim8.newAnimation(self.grid_eye_left("1-6", 1), 0.1)
	self.animations_eye.right_fly = anim8.newAnimation(self.grid_eye_right("6-1", 1), 0.1)
	self.animations_eye.projectile = anim8.newAnimation(self.grid_eye_projectile("1-8", 1), 0.1)

	self.animations = {}
	self.animations.left_roll = anim8.newAnimation(self.grid_left("12-1", 1), 0.05)
	self.animations.left_idle = anim8.newAnimation(self.grid_left("12-3", 2), 0.1)
	self.animations.left_run = anim8.newAnimation(self.grid_left("12-3", 3), 0.1)
	self.animations.left_attack = anim8.newAnimation(self.grid_left("12-3", 4), 0.1)
	self.animations.left_death = anim8.newAnimation(self.grid_left("12-3", 5), 0.1)
	self.animations.left_hit = anim8.newAnimation(self.grid_left("12-12", 6), 0.1)

	self.animations.right_roll = anim8.newAnimation(self.grid_right("1-12", 1), 0.05)
	self.animations.right_idle = anim8.newAnimation(self.grid_right("1-10", 2), 0.1)
	self.animations.right_run = anim8.newAnimation(self.grid_right("1-10", 3), 0.1)
	self.animations.right_attack = anim8.newAnimation(self.grid_right("1-10", 4), 0.1)
	self.animations.right_death = anim8.newAnimation(self.grid_right("1-10", 5), 0.1)
	self.animations.right_hit = anim8.newAnimation(self.grid_right("1-1", 6), 0.1)

	self.timer = 30

	self.s_hitbox = {}
	self.s_hitbox.x = self.x
	self.s_hitbox.y = self.y
	self.s_hitbox.width = 30
	self.s_hitbox.height = 30

	self.directions = {}
	self.directions.left = "left"
	self.directions.right = "right"

	_Esword = self.s_hitbox

	self.hitbox = {}
	self.hitbox.x = self.x
	self.hitbox.y = self.y
	self.hitbox.width = 20
	self.hitbox.height = 38


	_Ehitbox = self.hitbox

	self.hits = 3
	self.detect_range = 100

	self.death = false
	self.current_img = self.img_left
	self.current_anim = self.animations.left_idle

	self.attacking = false
	self.hit = false

	self.enemy_table = {}

	self.spawn_timer_knight = 2000
	self.spawn_speed = 200
	self.spawn_timer_eye = 3000
end

function enemies:update(dt)
	flux.update(dt)

	enemies:anim_timer(dt)
	enemies:locate_player(dt)
	enemies:hitbox_u(dt)
	enemies:attack(dt)
	enemies:projectile(dt)

	enemies:spawn_knight(dt)
	enemies:spawn_eye(dt)
	enemies:remove()


	for i, e in pairs(self.enemy_table) do
		e.current_anim:update(dt)

		if e.type == "eye" then
			for i, p in pairs(e.projectiles) do
				e.animations.projectile:update(dt)
			end
		end
	end
end

function enemies:draw()	
	love.graphics.push()
	love.graphics.scale(2, 2)

	for i, e in pairs(self.enemy_table) do
		
		if dev_mode == true then
			love.graphics.rectangle("line", e.hitbox.x / 2, e.hitbox.y / 2, e.hitbox.width, e.hitbox.height)
			if e.sword then
				love.graphics.rectangle("line", e.sword.x / 2, e.sword.y / 2, e.sword.width, e.sword.height)
			end
		end

		e.current_anim:draw(e.current_img, e.x / 2, e.y / 2)

		if e.type == "eye" then
			if dev_mode == true then
				love.graphics.print(e.projectiles_left, e.x / 2, e.y / 2)
			end
			for i, p in pairs(e.projectiles) do
				if dev_mode == true then
					love.graphics.rectangle("line", p.x / 2, p.y / 2, p.width, p.width)
				end
				
				if p.explode == false then
					love.graphics.draw(self.img_eye_projectiles, p.img, p.x / 2, p.y / 2)
				end

				if p.explode == true then
					e.animations.projectile:draw(self.img_eye_projectiles, p.x / 2, p.y / 2)
				end	
			end
		end
	end
		
	love.graphics.pop()
end

function enemies:anim_timer(dt)
	for i, e in pairs(self.enemy_table) do
		e.timer = e.timer - 100 * dt

		if e.timer <= 0 then
			e.timer = 0

			if e.type == "knight" then
				if e.current_anim == e.animations.left_attack then
					e.attacking = false
					e.current_anim = e.animations.left_idle
				end

				if e.current_anim == e.animations.right_attack then
					e.attacking = false
					e.current_anim = e.animations.right_idle
				end

				if e.current_anim == e.animations.left_hit and e.hit == true then
					e.attacking = false
					e.hit = false
					e.current_anim = e.animations.left_idle
				end

				if e.current_anim == e.animations.right_hit and e.hit == true then
					e.hit = false
					e.current_anim = e.animations.right_idle
				end

				if e.current_anim == e.animations.left_death and e.d_timer <= 0 then
					table.remove(self.enemy_table, i)
					player.score = player.score + 1
					player.replenish_health = player.replenish_health + 1
					menu.highscore = player.score
				end

				if e.current_anim == e.animations.right_death and e.d_timer <= 0 then
					table.remove(self.enemy_table, i)
					player.score = player.score + 1
					player.replenish_health = player.replenish_health + 1
					menu.highscore = player.score
				end
			end

			if e.type == "eye" then
				if e.current_anim == e.animations.left_hit and e.hit == true then
					e.attacking = false
					e.hit = false
					e.current_anim = e.animations.left_fly
				end

				if e.current_anim == e.animations.right_hit and e.hit == true then
					e.attacking = false
					e.hit = false
					e.current_anim = e.animations.right_fly
				end

				if e.current_anim == e.animations.left_death and e.d_timer <= 0 then
					table.remove(self.enemy_table, i)
					player.score = player.score + 1
					player.replenish_health = player.replenish_health + 1
				end

				if e.current_anim == e.animations.right_death and e.d_timer <= 0 then
					table.remove(self.enemy_table, i)
					player.score = player.score + 1
					player.replenish_health = player.replenish_health + 1
				end
			end
		end
	end
end

function enemies:locate_player(dt)
	if player.death == false then
		for i, e in pairs(self.enemy_table) do
			if (e.hitbox.x / 2) > _hitbox.x / 2  + _hitbox.width and e.hit == false then
				e.timer = 40
				e.current_direction = e.directions.left
				
				if e.type == "knight" then
					e.current_img = e.img_left
					e.current_anim = e.animations.left_idle
				end

				if (e.hitbox.x / 2) - self.detect_range <= (_hitbox.x / 2) + _hitbox.width and e.hit == false then
					e.current_direction = e.directions.left
					
					if e.type == "knight" then
						e.current_img = e.img_left
						e.current_anim = e.animations.left_run
					end

					if e.type == "eye" then
						e.current_img = e.img_left
						e.current_anim = e.animations.left_fly
					end

					e.x = e.x - 100 * dt
				end 
			end

			if (e.hitbox.x / 2) + e.hitbox.width < _hitbox.x / 2 and e.hit == false then
				e.timer = 40
				e.current_direction = e.directions.right
				
				if e.type == "knight" then
					e.current_img = e.img_right
					e.current_anim = e.animations.right_idle
				end

				if ((e.hitbox.x / 2) + e.hitbox.width) + self.detect_range >= (_hitbox.x / 2) and e.hit == false then
					e.current_direction = e.directions.right
					
					if e.type == "knight" then
						e.current_img = e.img_right
						e.current_anim = e.animations.right_run
					end


					if e.type == "eye" then
						e.current_img = e.img_right
						e.current_anim = e.animations.right_fly
					end

					e.x = e.x + 100 * dt
				end 
			end

			if CheckCollision(e.hitbox, player) then
				if e.hit == false then
					if e.current_direction == e.directions.left then
						e.current_img = e.img_left
						e.attacking = true
						
						if e.type == "knight" then
							e.current_anim = e.animations.left_attack
						end
					end
		
					if e.current_direction == e.directions.right then
						e.current_img = e.img_right
						if e.type == "knight" then
							e.current_anim = e.animations.right_attack
						end
						e.attacking = true
					end
				end
			end
		end
	end
end

function enemies:attack(dt)
	if player.death == false then
		for i, e in pairs(self.enemy_table) do
			if e.current_direction == e.directions.right then
				if e.sword then
					e.sword.x = e.x + 150
				end
			end
	
			if e.current_direction == e.directions.left then
				if e.sword then
					e.sword.x = e.x + 50
				end 
			end

			if e.sword then
				e.sword.y = 420
			end

			if player.rolling == false then
				if e.sword then
					if CheckCollision(e.sword, _hitbox) and e.hit == false then
	   					player.timer = 20
	   					player.health = player.health - 50 * dt

	   					if player.current_dir == player.directions.left then
	   						player.current_img = player.img_left
							player.current_anim = player.animations.left_hit
						end

						if player.current_dir == player.directions.right then
		   					player.current_img = player.img_right
							player.current_anim = player.animations.right_hit
						end
	   				end
	   			end
	   		end 
		end
	end
end

function enemies:hitbox_u(dt)
	for i, e in pairs(self.enemy_table) do
		e.hitbox.x = e.x + 90
		e.hitbox.y = e.y + 90
		if e.type == "eye" then
			e.fireball.x = e.x + 50
		end
	end
end

function enemies:spawn_knight(dt)
	self.spawn_timer_knight = self.spawn_timer_knight - self.spawn_speed * dt

	if self.spawn_timer_knight <= 0 then
		self.spawn_timer_knight = 2000
		enemy = {}	
		enemy.type = "knight"
		enemy.x = math.random(50, screenWidth - 80)
		enemy.y = 330
		enemy.width = 120
		enemy.height = 80
		enemy.hitbox = {}
		enemy.hitbox.x = enemy.x
		enemy.hitbox.y = enemy.y
		enemy.hitbox.width = 20
		enemy.hitbox.height = 38
		enemy.sword = {}
		enemy.sword.x = enemy.x
		enemy.sword.y = enemy.y 
		enemy.sword.width = 5
		enemy.sword.height = 5
		enemy.directions = {}
		enemy.directions.left = "left"
		enemy.directions.right = "right"
		enemy.current_direction = enemy.directions.left
		enemy.attacking = false
		enemy.hit = false
		enemy.death = false
		enemy.d_timer = 50
		
		enemy.hits = 3
		enemy.timer = 30
		
		enemy.img_left = self.img_left
		enemy.img_right = self.img_right

		enemy.grid_left = anim8.newGrid(enemy.width, enemy.height, enemy.img_left:getWidth(), enemy.img_left:getHeight())
		enemy.grid_right = anim8.newGrid(enemy.width, enemy.height, enemy.img_right:getWidth(), enemy.img_right:getHeight())

		enemy.animations = {}
		enemy.animations.left_roll = anim8.newAnimation(enemy.grid_left("12-1", 1), 0.05)
		enemy.animations.left_idle =  anim8.newAnimation(enemy.grid_left("12-3", 2), 0.1)
		enemy.animations.left_run = anim8.newAnimation(enemy.grid_left("12-3", 3), 0.1)
		enemy.animations.left_attack = anim8.newAnimation(enemy.grid_left("12-3", 4), 0.1)
		enemy.animations.left_death = anim8.newAnimation(enemy.grid_left("12-3", 5), 0.1)
		enemy.animations.left_hit = anim8.newAnimation(enemy.grid_left("12-12", 6), 0.1)

		enemy.animations.right_roll = anim8.newAnimation(enemy.grid_right("1-12", 1), 0.05)

		enemy.animations.right_idle = anim8.newAnimation(enemy.grid_right("1-10", 2), 0.1)
		enemy.animations.right_run = anim8.newAnimation(enemy.grid_right("1-10", 3), 0.1)
		enemy.animations.right_attack = anim8.newAnimation(enemy.grid_right("1-10", 4), 0.1)
		enemy.animations.right_death = anim8.newAnimation(enemy.grid_right("1-10", 5), 0.1)
		enemy.animations.right_hit = anim8.newAnimation(enemy.grid_right("1-1", 6), 0.1)

		enemy.current_img = enemy.img_left
		enemy.current_anim = enemy.animations.left_idle

		table.insert(self.enemy_table, enemy)
	end
end

function enemies:spawn_eye(dt)
	self.spawn_timer_eye = self.spawn_timer_eye - self.spawn_speed * dt

	if self.spawn_timer_eye <= 0 then
		self.spawn_timer_eye = 3000
		eye = {}	
		eye.type = "eye"
		eye.x = math.random(50, screenWidth - 80)
		eye.y = 200
		eye.width = 150
		eye.height = 150
		eye.hitbox = {}
		eye.hitbox.x = eye.x
		eye.hitbox.y = eye.y
		eye.hitbox.width = 50
		eye.hitbox.height = 50
		eye.fireball = {}
		eye.fireball.x = eye.x
		eye.fireball.y = eye.y
		eye.fireball.width = 48
		eye.fireball.height = 48
		eye.fireball.img = self.img_projectile
		eye.fireball.explode = false
		eye.directions = {}
		eye.directions.left = "left"
		eye.directions.right = "right"
		eye.current_direction = eye.directions.left
		eye.attacking = false
		eye.hit = false
		eye.death = false
		eye.d_timer = 35
		
		eye.hits = 1
		eye.timer = 30
		
		eye.img_left = self.img_eye_left
		eye.img_right = self.img_eye_right
		eye.img_eye_projectiles = self.img_eye_projectiles

		eye.grid_eye_left = anim8.newGrid(eye.width, eye.height, eye.img_left:getWidth(), eye.img_left:getHeight())
		eye.grid_eye_right = anim8.newGrid(eye.width, eye.height, eye.img_right:getWidth(), eye.img_right:getHeight())
		eye.grid_eye_projectile = anim8.newGrid(48, 48, eye.img_eye_projectiles:getWidth(), eye.img_eye_projectiles:getHeight())

		eye.animations = {}
		eye.animations.left_attack = anim8.newAnimation(eye.grid_eye_left("8-1", 1), 0.1) 
		eye.animations.left_fly = anim8.newAnimation(eye.grid_eye_left("6-3", 2), 0.1)
		eye.animations.left_hit = anim8.newAnimation(eye.grid_eye_left("8-5", 3), 0.1)
		eye.animations.left_death = anim8.newAnimation(eye.grid_eye_left("8-5", 4), 0.1)

		eye.animations.right_attack = anim8.newAnimation(eye.grid_eye_right("1-8", 1), 0.1) 
		eye.animations.right_fly = anim8.newAnimation(eye.grid_eye_right("1-6", 2), 0.1)
		eye.animations.right_hit = anim8.newAnimation(eye.grid_eye_right("1-4", 3), 0.1)
		eye.animations.right_death = anim8.newAnimation(eye.grid_eye_right("1-4", 4), 0.1)

		eye.animations.projectile = anim8.newAnimation(eye.grid_eye_projectile("1-8", 1), 0.1)

		eye.current_img = eye.img_left
		eye.current_anim = eye.animations.left_fly

		eye.states = {
					  fly = "fly",
					  attack = "attack"
					 }

		eye.current_state = eye.states.fly

		eye.projectiles = {}
		eye.set_timer = 500
		eye.projectile_timer = 500
		eye.reset_timer = 500500
		eye.projectiles_left = 3
		eye.randomX = math.random(1, 3)

		table.insert(self.enemy_table, eye)
	end
end

function enemies:projectile(dt)
	for i, e in pairs(enemies.enemy_table) do
		if e.type == "eye" then
			local oldX = e.x
			local oldY = e.y

			e.set_timer = e.set_timer - 100 * dt

			if e.set_timer <= 0 then
				e.set_timer = 0
				if e.randomX == 1 then
					flux.to(e, 2, {x = 0, y = 10})
				elseif e.randomX == 2 then
					flux.to(e, 2, {x = 650, y = 10})
				elseif e.randomX == 3 then
					flux.to(e, 2, {x = 300, y = 10})
				end
			end
				
			if e.set_timer <= 0 then
				e.projectile_timer = e.projectile_timer - 200 * dt 
			end

			if e.projectile_timer <= 0 and e.projectiles_left > 0 then
				
				if e.current_direction == e.directions.left then
					e.current_anim = e.animations.left_attack
				end

				if e.current_direction == e.directions.right then
					e.current_anim = e.animations.right_attack
				end

				e.projectile_timer = 500
				e.projectiles_left = e.projectiles_left - 1

				table.insert(e.projectiles, {x = e.x, y = e.y, width = 48, height = 48, explode = false, img = e.fireball.img})
			end
			
			for i1, p in pairs(e.projectiles) do
				p.y = p.y + 200 * dt
				
				if p.y >= screenHeight then
					table.remove(e.projectiles, i1)
				end
				if CheckCollision(_sword, p) then
					if player.attacking == true then
						p.explode = true
						table.remove(e.projectiles, i1)
					end
				end

				if CheckCollision(p, _hitbox) then
					p.explode = true
					player.health = player.health - 20 * dt 
				end
			end

			if e.projectiles_left <= 0 then
				flux.to(e, 2, {x = oldX, y = 200})
				e.set_timer = 500
				e.projectile_timer = 500
				e.reset_timer = e.reset_timer - 200 * dt
				if e.reset_timer <= 0 then
					if e.current_direction == e.directions.left then
						e.current_anim = e.animations.left_fly
					end
					if e.current_direction == e.directions.right then
						e.current_anim = e.animations.right_fly
					end
					e.projectiles_left = 3
				end
			end
		end
	end
end

function enemies:remove()
	if player.death == true then
		for i, e in pairs(self.enemy_table) do
			table.remove(self.enemy_table, i)
		end
	end
end