local buttons = require "libraries/buttons"

menu = {}

function menu:load()

	self.font = love.graphics.newFont("assets/fonts/PirataOne-Regular.ttf", 15)

	love.graphics.setFont(self.font)

	sounds = {
			  hit = love.audio.newSource("assets/sounds/slash.mp3"),
			  swing = love.audio.newSource("assets/sounds/swing.mp3"),
			  thud = love.audio.newSource("assets/sounds/thud.mp3"),
			  swoosh = love.audio.newSource("assets/sounds/swoosh.mp3")
			 }

	music = {
			 main_menu = love.audio.newSource("assets/sounds/sawsquarenoise - El Pastor Cordial.mp3"),
			 in_game = love.audio.newSource("assets/sounds/sawsquarenoise - Tsuneni.mp3")
			}

	music.main_menu:setLooping()
	music.main_menu:play()

	self.states = {
					menu = "menu",
					action = "action"	
				  }

	self.current_state = self.states.menu

	self.buttons = { 
					 start = { x = 344, y = 300, width = 100, height = 50, txt = "Start", type = "line"},
				     settings = {x = 344, y = 390, width = 100, height = 50, txt = "Highscore: ", type = "line"},
					 quit = {x = 344, y = 480, width = 100, height = 50, txt = "quit", type = "line"},
					 fullscreen = {x = 50, y = 480, width = 100, height = 50, txt = "fullscreen: ", type = "line", toggle = "off"}
			  	   }
	self.logo = {x = 230 , y = 50, width = 350, height = 200, img = love.graphics.newImage("assets/backgrounds/logo.png")}

	self.shuffle_tweak = 10

	self.highscore = 0

	_mousebox = {x = 0, y = 0, width = 10, height = 10}
	_start_m = self.buttons.start
	_settings = self.buttons.settings
	_quit = self.buttons.quit
	_fullscreen = self.buttons.fullscreen

	sawsquarenoise = buttons:new(10, 10, "https://freemusicarchive.org/music/sawsquarenoise/", l, t)
end

function menu:update(dt)
	_mousebox.x = love.mouse.getX()
	_mousebox.y = love.mouse.getY()

	if CheckCollisions(_mousebox, _start_m) then 
		_start_m.type = "fill"
	else
		_start_m.type = "line"
	end

	if CheckCollisions(_mousebox, _settings) then
		_settings.type = "fill"
	else
		_settings.type = "line"
	end

	if CheckCollisions(_mousebox, _quit) then 
		_quit.type = "fill"
	else
		_quit.type = "line"
	end

	if CheckCollisions(_mousebox, _fullscreen) then 
		_fullscreen.type = "fill"
	else
		_fullscreen.type = "line"
	end
	fulls()
end

function menu:draw()
	--[[if CheckCollisions(_mousebox, self.logo) then
		love.graphics.setColor(255, 100, 100, 100)
	else
		love.graphics.setColor(255, 255, 255)
	end
	]]--

	love.graphics.draw(self.logo.img, self.logo.x, self.logo.y)

	love.graphics.setColor(255, 255, 255)
	love.graphics.rectangle(_start_m.type, _start_m.x, _start_m.y, _start_m.width, _start_m.height)

	love.graphics.setColor(0, 0, 0)
	love.graphics.print(_start_m.txt, _start_m.x + 32, _start_m.y + 15)

	love.graphics.setColor(255, 255, 255)
	love.graphics.rectangle(_settings.type, _settings.x, _settings.y, _settings.width, _settings.height)
	
	love.graphics.setColor(0, 0, 0)
	love.graphics.print(_settings.txt.."\n".."       "..self.highscore, _settings.x, _settings.y + 15)

	love.graphics.setColor(255, 255, 255, 100)

	love.graphics.rectangle("fill", _settings.x + 200, _settings.y - 100, 200, 160)
	love.graphics.setColor(0, 0, 0)
	love.graphics.print("Press O to mute audio\n\nPress P to turn on audio\n\nM for menu", _settings.x + 200, _settings.y - 100)

	love.graphics.setColor(255, 255, 255)
	love.graphics.rectangle(_quit.type, _quit.x, _quit.y, _quit.width, _quit.height)

	love.graphics.setColor(0, 0, 0)
	love.graphics.print(_quit.txt, _quit.x + 33, _quit.y + 15)

	love.graphics.setColor(255, 255, 255)
	love.graphics.rectangle(_fullscreen.type, _fullscreen.x, _fullscreen.y, _fullscreen.width, _fullscreen.height)

	love.graphics.setColor(0, 0, 0)
	love.graphics.print(_fullscreen.txt.._fullscreen.toggle, _fullscreen.x + 8, _fullscreen.y + 15)

	love.graphics.setColor(255, 255, 255)

	sawsquarenoise:draw()
end

function love.mousepressed(x, y, b, istouch)
	if menu.current_state == menu.states.menu then 
		if CheckCollisions(_mousebox, _start_m) then 
			if b == "l" then
				menu.current_state = menu.states.action
				maps.parallax_7.x, maps.parallax_7.y = 0, 0
				maps.parallax_8.x, maps.parallax_8.y = 800, 0
				maps.platform.x, maps.platform.y = 380, 340
			end
		end

		if CheckCollisions(_mousebox, _quit) then 
			if b == "l" then 
				love.event.quit()
			end
		end

		if CheckCollisions(_mousebox, _fullscreen) and _fullscreen.toggle == "off" then
			if b == "l" then
				_fullscreen.toggle = "on"
			end
		elseif CheckCollisions(_mousebox, _fullscreen) and _fullscreen.toggle == "on" then
			if b == "l" then
				_fullscreen.toggle = "off"
			end
		end

		if CheckCollisions(_mousebox, menu.logo) then 
			if b == "l" then 
				menu.eggs_toggle = true
			end
		end
	end

	sawsquarenoise:mousepressed(x, y, b)
end

function fulls()
	if _fullscreen.toggle == "on" then
		love.window.setFullscreen(true, "normal")
	elseif _fullscreen.toggle == "off" then
		love.window.setFullscreen(false, "normal")
		love.window.setMode(800, 600, {resizable = false, vsync = false, minwidth = 800, minheight = 600})
	end
end

function highscore()
	if player.score > menu.highscore then
		menu.highscore = player.score
	end
end

function CheckCollisions(a, b)
	return a.x < b.x + b.width and
		   b.x < a.x + a.width and
		   a.y < b.y + b.height and
		   b.y < a.y + a.height
end