local anim8 = require "libraries/anim8"

player = {}

function player:load()
	love.graphics.setDefaultFilter("nearest", "nearest")
	screenWidth = love.graphics.getWidth()
	screenHeight = love.graphics.getHeight()

	self.width = 120
	self.height = 80

	self.x = (screenWidth - self.width) / 2
	self.y = (screenHeight - self.height) / 2
	self.speed = 10
	self.xvel = 0
	self.yvel = 0
	self.friction = 3.5
	self.jumping = true
	self.max_jump = 15
	self.max_height = 430
	self.roll_distance = 10
	self.health = 100
	self.portrait = love.graphics.newImage("assets/player/portrait.png")

	self.img_left = love.graphics.newImage("assets/player/player_left.png")
	self.img_right = love.graphics.newImage("assets/player/player_right.png")

	self.grid_left = anim8.newGrid(self.width, self.height, self.img_left:getWidth(), self.img_left:getHeight())
	self.grid_right = anim8.newGrid(self.width, self.height, self.img_right:getWidth(), self.img_right:getHeight())

	self.animations = {}
	self.animations.left_roll = anim8.newAnimation(self.grid_left("12-1", 1), 0.05)
	self.animations.left_idle = anim8.newAnimation(self.grid_left("12-3", 2), 0.1)
	self.animations.left_run = anim8.newAnimation(self.grid_left("12-3", 3), 0.1)
	self.animations.left_attack1 = anim8.newAnimation(self.grid_left("12-9", 4), 0.1)
	self.animations.left_attack2 = anim8.newAnimation(self.grid_left("12-7", 5), 0.1)
	self.animations.left_turn = anim8.newAnimation(self.grid_left("12-10", 6), 0.1)
	self.animations.left_jump = anim8.newAnimation(self.grid_left("12-10", 7), 0.1)
	self.animations.left_hit = anim8.newAnimation(self.grid_left("12-12", 8), 0.1)
	self.animations.left_death = anim8.newAnimation(self.grid_left("12-3", 9), 0.1)
	self.animations.left_dash = anim8.newAnimation(self.grid_left("12-11", 10), 0.05)
	self.animations.left_slide = anim8.newAnimation(self.grid_left("12-9", 11), 0.1)

	self.animations.right_roll = anim8.newAnimation(self.grid_right("1-12", 1), 0.05)
	self.animations.right_idle = anim8.newAnimation(self.grid_right("1-10", 2), 0.1)
	self.animations.right_run = anim8.newAnimation(self.grid_right("1-10", 3), 0.1)
	self.animations.right_attack1 = anim8.newAnimation(self.grid_right("1-4", 4), 0.1)
	self.animations.right_attack2 = anim8.newAnimation(self.grid_right("1-6", 5), 0.1)
	self.animations.right_turn = anim8.newAnimation(self.grid_right("1-3", 6), 0.1)
	self.animations.right_jump = anim8.newAnimation(self.grid_right("1-3", 7), 0.1)
	self.animations.right_hit = anim8.newAnimation(self.grid_right("1-1", 8), 0.1)
	self.animations.right_death = anim8.newAnimation(self.grid_right("1-10", 9), 0.1)
	self.animations.right_dash = anim8.newAnimation(self.grid_right("1-2", 10), 0.05)
	self.animations.right_slide = anim8.newAnimation(self.grid_right("1-4", 11), 0.1)

	self.directions = {}
	self.directions.left = "left"
	self.directions.right = "right"
	
	self.current_dir = self.directions.right

	self.current_img = self.img_right
	self.current_anim = self.animations.right_idle

	self.attack_counter = 1
	self.attacking = false
	self.rolling = false
	self.death = false
	self.canDash = false
	self.slide = false
	self.timer = 30
	self.dash_timer = 100

	self.s_hitbox = {}
	self.s_hitbox.x = self.x
	self.s_hitbox.y = self.y
	self.s_hitbox.width = 30
	self.s_hitbox.height = 30

	self.hitbox = {}
	self.hitbox.x = self.x
	self.hitbox.y = self.y
	self.hitbox.width = 20
	self.hitbox.height = 38
	self.score = 0
	self.replenish_health = 0
	self.onPlatform = false

	_sword = self.s_hitbox
	_hitbox = self.hitbox

	enemy = {
			 x = 400,
		     y = 500,
			 width = 40,
			 height = 40,
			 vis = true
			}
end

function player:update(dt)
	self.dash_timer = self.dash_timer - 100 * dt

	player:movement(dt)
	player:jump(dt)
	player:anim_timer(dt)
	player:attack(dt)
	player:hitbox_u(dt)
	player:boundary(dt)
	player:death_anim()
	player:dash_u(dt)
	player:score_events()

	self.current_anim:update(dt)
	
	highscore()
end

function player:draw()
	love.graphics.push()
	love.graphics.scale(2, 2)
	
	if dev_mode == true then
		love.graphics.rectangle("line", _hitbox.x / 2, _hitbox.y / 2 , _hitbox.width, _hitbox.height)
		love.graphics.rectangle("line", _sword.x / 2, _sword.y / 2, _sword.width, _sword.height)
	end

	self.current_anim:draw(self.current_img, self.x / 2, self.y / 2)
	
	love.graphics.pop() 

	love.graphics.print("Health: "..math.floor(self.health), 240)
	if dev_mode == true then
		love.graphics.print("Attacking: "..tostring(self.attacking))
		love.graphics.print("jumping: "..tostring(self.jumping), 500)
		love.graphics.print("canDash: "..tostring(self.canDash), 300)
		love.graphics.print("playerX: "..math.floor(player.x), player.x, player.y)
		love.graphics.print("playerY: "..math.floor(player.y), player.x + 100, player.y)
	end

	love.graphics.print("score: "..math.floor(self.score), 600)
	
	if self.death == true then
		love.graphics.print("R to restart", (screenWidth - 50) / 2, (screenHeight - 30) / 2)
	end
	
	if self.health <= 25 then
		love.graphics.setColor(255, 100, 100)
	else
		love.graphics.setColor(100, 255, 100)
	end
	love.graphics.rectangle("fill", 120, 10, self.health, 10)
	
	
	love.graphics.setColor(255, 255, 255)

	love.graphics.draw(self.portrait, 10, 10)
end

function player:movement(dt)
	if (self.current_anim ~= self.animations.right_attack1 or self.current_anim ~= self.animations.right_attack1 or self.canDash == false) then
		self.yvel = self.yvel + (75 / 2) * dt
	end
	
	self.yvel = self.yvel * (1 - math.min(dt * self.friction, 1))
	self.xvel = self.xvel * (1 - math.min(dt * self.friction, 1))	

	if self.death ~= true then
		if love.keyboard.isDown("left") and self.xvel > -100 and self.rolling == false and self.jumping == false then
			self.xvel = self.xvel - self.speed * dt
			self.current_anim = self.animations.left_run
			self.current_img = self.img_left
			self.current_dir = self.directions.left
			self.attacking = false
		end	

		if love.keyboard.isDown("right") and self.xvel < 100 and self.rolling == false and self.jumping == false then
			self.xvel = self.xvel + self.speed * dt
			self.current_anim = self.animations.right_run
			self.current_img = self.img_right
			self.current_dir = self.directions.right
			self.attacking = false
		end
	
		if self.y > self.max_height - 32 - 64 then
			self.jumping = false
			self.y = self.max_height - 32 - 64
			self.yvel = 0
		end
	
		self.x = self.x + self.xvel
		self.y = self.y + self.yvel
	end
end

function player:jump(dt)
	if love.keyboard.isDown("left") then
		if love.keyboard.isDown("x") and self.jumping == false then
			self.timer = 30
			self.yvel = self.yvel - self.max_jump
			self.xvel = self.xvel - 5
			self.jumping = true
			self.current_img = self.img_left
			self.current_anim = self.animations.left_jump
		end
	end

	if love.keyboard.isDown("right") then
		if love.keyboard.isDown("x") and self.jumping == false then 
			self.timer = 30
			self.yvel = self.yvel - self.max_jump
			self.xvel = self.xvel + 5
			self.jumping = true
			self.current_img = self.img_right
			self.current_anim = self.animations.right_jump
		end
	end
	
	if love.keyboard.isDown("x") and self.jumping == false then
		self.timer = 20
		self.yvel = self.yvel - self.max_jump
		self.jumping = true

		if self.current_dir == self.directions.left then
			self.current_img = self.img_left
			self.current_anim = self.animations.left_jump
		end

		if self.current_dir == self.directions.right then
			self.current_img = self.img_right
			self.current_anim = self.animations.right_jump
		end
	end
end

function player:anim_timer(dt)
 	self.timer = self.timer - 100 * dt

	if self.timer <= 0 then
		self.timer = 0
		self.rolling = false

		if self.current_anim == self.animations.left_turn then
			self.current_anim = self.animations.left_idle
		end
		if self.current_anim == self.animations.right_turn then
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.left_attack1 then
			self.attacking = false
			self.attack_counter = 2
			self.current_anim = self.animations.left_idle
		end

		if self.current_anim == self.animations.right_attack1 then
			self.attacking = false
			self.attack_counter = 2
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.left_attack2 then
			self.attacking = false
			self.attack_counter = 1
			self.current_anim = self.animations.left_idle
		end

		if self.current_anim == self.animations.right_attack2 then
			self.attacking = false
			self.attack_counter = 1
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.left_roll then
			self.current_anim = self.animations.left_idle
		end

		if self.current_anim == self.animations.right_roll then
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.right_jump then
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.left_jump then
			self.current_anim = self.animations.left_idle
		end

		if self.current_anim == self.animations.right_dash then
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.left_dash then
			self.current_anim = self.animations.left_idle
		end

		if self.current_anim == self.animations.right_slide then
			self.current_anim = self.animations.right_idle
		end

		if self.current_anim == self.animations.left_slide then
			self.current_anim = self.animations.left_idle
		end
	end
end

function player:key_pressed(key, isrepeat)
	if self.death ~= true then
		if (self.current_anim == self.animations.right_idle or 
			self.current_anim == self.animations.right_turn or 
	    	self.current_anim == self.animations.right_run) then
		
			if key == "z" and self.attack_counter == 1 and self.timer <= 0 then
				love.audio.play(sounds.swing)
				self.timer = 40
				self.attacking = true
				self.current_img = self.img_right
				self.current_anim = self.animations.right_attack1
				self.current_anim:gotoFrame(1)
			end

			if key == "z" and self.attack_counter == 2 and self.timer <= 0 then
				love.audio.play(sounds.swing)
				self.timer = 40
				self.attacking = true
				self.current_img = self.img_right
				self.current_anim = self.animations.right_attack2
				self.current_anim:gotoFrame(1)
			end
		
			if key == " " and self.rolling == false then 
				self.rolling = true
				self.roll_counter = 0
				self.timer = 60
				self.current_img = self.img_right
				self.current_anim = self.animations.right_roll
				self.xvel = self.xvel + self.roll_distance
				self.current_anim:gotoFrame(1)
			end
		end

		if (self.current_anim == self.animations.left_idle or 
			self.current_anim == self.animations.left_turn or 
			self.current_anim == self.animations.left_run) then

		
				if key == "z" and self.attack_counter == 1 and self.timer <= 0 then
					love.audio.play(sounds.swing)
					self.timer = 40
					self.attacking = true
					self.current_img = self.img_left
					self.current_anim = self.animations.left_attack1
					self.current_anim:gotoFrame(1)
				end

				if key == "z" and self.attack_counter == 2 and self.timer <= 0 then
					love.audio.play(sounds.swing)
					self.timer = 40
					self.attacking = true
					self.current_img = self.img_left
					self.current_anim = self.animations.left_attack2
					self.current_anim:gotoFrame(1)
				end
			
			if key == " " and self.rolling == false then
				self.rolling = true
				self.timer = 60
				self.current_img = self.img_left
				self.current_anim = self.animations.left_roll
				self.current_anim:gotoFrame(1)
				self.xvel = self.xvel - self.roll_distance
			end
		end
	end
end

function player:key_released(key)
	if self.death ~= true then
		if key == "left" and self.rolling == false then
			self.timer = 25
			self.current_img = self.img_left
			self.current_anim = self.animations.left_turn
		end

		if key == "right" and self.rolling == false then
			self.timer = 25
			self.current_img = self.img_right
			self.current_anim = self.animations.right_turn
		end
	end
end

function player:attack(dt)
	if self.current_dir == self.directions.right then
		_sword.x = self.x + 170
	end
	
	if self.current_dir == self.directions.left then
		_sword.x = self.x + 10
	end

	_sword.y = self.y + 100
	
	for i, e in pairs(enemies.enemy_table) do
		if self.attacking == true then
			if e.type == "knight" or e.type == "eye" then
				if CheckCollision(_sword, e.hitbox) then
					love.audio.play(sounds.hit)
	   				e.hit = true
	   				e.timer = 20
	   				e.hits = e.hits - 3 * dt	   					

	   				if e.current_direction == e.directions.left then
	   					e.current_img = e.img_left
						e.current_anim = e.animations.left_hit
					end

					if e.current_direction == e.directions.right then
	   					e.current_img = e.img_right
						e.current_anim = e.animations.right_hit
					end
		   		end
			end
		end
		if self.canDash == true then
			if e.type == "eye" then
				if CheckCollision(_hitbox, e.hitbox) then
					love.audio.play(sounds.thud)
					e.hit = true
	   				e.timer = 40
	   				e.hits = e.hits - 5 * dt	   		
					
					if e.current_direction == e.directions.left then
	   					e.current_img = e.img_left
						e.current_anim = e.animations.left_hit
					end

					if e.current_direction == e.directions.right then
	   					e.current_img = e.img_right
						e.current_anim = e.animations.right_hit
					end
				end
			end
		end
	end
end

function player:hitbox_u(dt)
	_hitbox.x = self.x + 90
	_hitbox.y = self.y + 90

	for i, e in pairs(enemies.enemy_table) do
		if e.hits <= 0 then
			e.d_timer = e.d_timer - 50 * dt
			e.death = true
			e.hits = 0
			
			if e.current_direction == e.directions.left then
				if e.type == "knight" then
		   			e.current_img = e.img_left
					e.current_anim = e.animations.left_death
				end
				
				if e.type == "eye" then
		   			e.current_img = e.img_left
					e.current_anim = e.animations.left_death
				end 
			end
			
			if e.current_direction == e.directions.right then
	   			if e.type == "knight" then
	   				e.current_img = e.img_right
					e.current_anim = e.animations.right_death
				end

				if e.type == "eye" then
		   			e.current_img = e.img_right
					e.current_anim = e.animations.right_death
				end
			end
		end  
	end
end

function player:boundary(dt)
	if self.x + self.width >= screenWidth then
		self.xvel = 0
		self.x = screenWidth - self.width 
	end
	if self.x <= 0 then
		self.xvel = 0
		self.x = 0 
	end
end

function player:death_anim()
	if self.health <= 0 then
		self.health = 0
		self.death = true
	end

	if self.health <= 0 and self.death == true and self.rolling == false then
		if self.current_dir == self.directions.left then
			self.current_anim = self.animations.left_death
		end

		if self.current_dir == self.directions.right then
			self.current_anim = self.animations.right_death
		end
	end
end

function player:restart(key)
	if key == "r" then
		self.health = 100
		self.death = false
		self.current_img = self.img_right
		self.current_anim = self.animations.left_idle
		self.score = 0
		self.replenish_health = 0
	end
end

function player:dash_u(dt)
	if self.canDash == true or self.slide == true then
		self.dash_timer = self.dash_timer - 100 * dt

		if self.dash_timer <= 0 then
			self.dash_timer = 0
			self.canDash = false
			self.slide = false
		end
	end

	if self.canDash == true or self.slide == true then
		love.audio.play(sounds.swoosh)
		if self.current_dir == self.directions.left then
			self.x = self.x - 450 * dt
			if self.canDash == true then
				self.current_img = self.img_left
				self.current_anim = self.animations.left_dash
			end

			if self.slide == true then
				self.current_img = self.img_left
				self.current_anim = self.animations.left_slide
			end
		end

		if self.current_dir == self.directions.right then
			self.x = self.x + 450 * dt
			if self.canDash == true then
				self.current_img = self.img_right
				self.current_anim = self.animations.right_dash
			end

			if self.slide == true then
				self.current_img = self.img_right
				self.current_anim = self.animations.right_slide
			end
		end		
	end

	for i, e in pairs(enemies.enemy_table) do
		if self.slide == true then
			if CheckCollision(_hitbox, e.hitbox) then
				love.audio.play(sounds.thud)
				e.hit = true
	   			e.timer = 20
	   			e.hits = e.hits - 3 * dt	   					

	   			if e.current_direction == e.directions.left then
	   				e.current_img = e.img_left
					e.current_anim = e.animations.left_hit
				end

				if e.current_direction == e.directions.right then
	   				e.current_img = e.img_right
					e.current_anim = e.animations.right_hit
				end
			end
		end
	end
end

function player:dash(key)
	if self.death ~= true then
		if key == "c" and self.rolling == false and self.slide == false then
			self.canDash = true
			self.dash_timer = 50
		end

		if key == "v" and self.rolling == false and self.canDash == false then
			self.slide = true
			self.dash_timer = 50
		end
	end
end

function player:score_events()
	if self.replenish_health >= 10 then
		self.health = 100
		self.replenish_health = 0
	end
end

function CheckCollision(a, b)
	return a.x / 2  < b.x / 2 + b.width and
		   b.x / 2 < a.x / 2 + a.width and
		   a.y / 2  < b.y / 2 + b.height and
		   b.y / 2 < a.y / 2 + a.height
end